# -*- coding: utf-8 -*-

"""Test utils for processing numeric literals."""

import unittest

class NumericLiteralsUtilsTests(unittest.TestCase):
    """Class for testing utils for processing numeric literals."""

    def test_create_matix_of_literals(self):
        """."""
        self.assertEqual(1,1)